==> SD35B_0.03.0_src.tar.gz: Sd35b/decoder/CDPlayer_BeOS.cpp <==
/*
 * CDPlayer_BeOS.cpp - "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * Copyright (C) 1999 Atsushi Mizuguchi  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * 
 *      99/05/24 KISAKA Toshihiro       modify for mwcc compatiblility
 *
 */

#include "stdafx.h"
#include "CDPlayer_BeOS.h"
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <Directory.h>
#include <Path.h>
#include <errno.h>
#include "scsi.h"
#include "scsiprobe_driver.h"

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
using namespace ALSD;
using namespace ALSD_BeOS;
using namespace ALSD_CDP_BeOS;
#endif
// "dev/disk"
int try_dir( const char *directory )
{ 
	BDirectory dir; 
	dir.SetTo(directory); 
	if(dir.InitCheck() != B_NO_ERROR) { 
		return false; 
	} 
	dir.Rewind(); 
	BEntry entry; 
	while(dir.GetNextEntry(&entry) >= 0) { 
		BPath path; 
		const char *name; 
		entry_ref e; 
		
		if(entry.GetPath(&path) != B_NO_ERROR) 
			continue; 
		name = path.Path(); 
		
		if(entry.GetRef(&e) != B_NO_ERROR) 
			continue; 

		if(entry.IsDirectory()) { 
			if(strcmp(e.name, "floppy") == 0) 
				continue; // ignore floppy (it is not silent) 
			int devfd = try_dir(name);
			if(devfd >= 0)
				return devfd;
		} else { 
			int devfd; 
			device_geometry g; 

			if(strcmp(e.name, "raw") != 0) 
				continue; // ignore partitions 

			devfd = open(name, O_RDONLY); 
			if(devfd < 0) 
				continue; 

			if(ioctl(devfd, B_GET_GEOMETRY, &g, sizeof(g)) >= 0) {
				if(g.device_type == B_CD)
				{ 
					return devfd;
				}
			}
			close(devfd);
		} 
	}
	return B_ERROR;
}

/**********************************************************************
	RXgNV
**********************************************************************/
CCDPlayer_BeOS::CCDPlayer_BeOS()
{
	m_bEnd			= false;
	m_NowTrack= 0;
	m_NowLoop = 0;
	m_Devicefd = try_dir( "/dev/disk" );

	// XbhN
}

CCDPlayer_BeOS::~CCDPlayer_BeOS()
{
	m_bEnd = true;
//	if( m_hThread ){
//		::snooze(500);
//		::TerminateThread( m_hThread, 0 );
//	}
}

/**********************************************************************
	gbN̎擾
**********************************************************************/
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
long CCDPlayer_BeOS::GetPlayTrack( ALSD::stCDPTrackInfo *_lpTrack )
#else
long CCDPlayer_BeOS::GetPlayTrack( stCDPTrackInfo *_lpTrack )
#endif
{
	return 0;
}

/**********************************************************************
	Đ(gbNOŒ~)
**********************************************************************/
long CCDPlayer_BeOS::Play( long _Track, int _Loop )
{
	m_NowTrack = _Track;
	m_NowLoop  = _Loop;

	return 0;
}

/**********************************************************************
	ĐThread
**********************************************************************/
int32 CCDPlayer_BeOS::PlayFunc( void *_lpVoid )
{
	CCDPlayer_BeOS *lpCDP = (CCDPlayer_BeOS *)_lpVoid;

	// foCX̃I[v

//	while( lpCDP->IsDestroy() ){
//	}
}

/**********************************************************************
	Ԃ̎擾
**********************************************************************/
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
ALSD_CDP_BeOS::CDState CCDPlayer_BeOS::GetState( void )
#else
CDState CCDPlayer_BeOS::GetState( void )
#endif
{
#if 0
	if( m_Devicefd == B_ERROR )return NoCD;

	// CD邩H
	status_t media_status = B_DEV_NO_MEDIA;
	ioctl(devicefd, B_GET_MEDIA_STATUS, &media_status, sizeof(media_status));
	if (media_status != B_NO_ERROR){
		return NoCD;
	}

	// ʒu擾
	scsi_position pos;
	status_t result = ioctl(devicefd, B_SCSI_GET_POSITION, &pos);

	//
	if( result != B_NO_ERROR ){
		//
		return NoCD;
	}else if( (!pos.position[1]) || (pos.position[1] >= 0x13) ||
		((pos.position[1] == 0x12) && (!pos.position[6])) ){
		//
		return Stopped;
	}else if (pos.position[1] == 0x11){
		//
		return Playing;
	}else{
		//
		return Paused;
	}
#endif
	return NoCD;
}

==> SD35B_0.03.2_src.tar.gz: Sd35b/decoder/CDPlayer_BeOS.cpp <==
/*
 * CDPlayer_BeOS.cpp - "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * Copyright (C) 1999 Atsushi Mizuguchi  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * 
 *      99/05/24 KISAKA Toshihiro       modify for mwcc compatiblility
 *
 */

#include "stdafx.h"
#include "CDPlayer_BeOS.h"
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <Directory.h>
#include <Path.h>
#include <errno.h>
#include "scsi.h"
#include "scsiprobe_driver.h"

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
using namespace ALSD;
using namespace ALSD_BeOS;
using namespace ALSD_CDP_BeOS;
#endif
// "dev/disk"
int try_dir( const char *directory )
{ 
	BDirectory dir; 
	dir.SetTo(directory); 
	if(dir.InitCheck() != B_NO_ERROR) { 
		return false; 
	} 
	dir.Rewind(); 
	BEntry entry; 
	while(dir.GetNextEntry(&entry) >= 0) { 
		BPath path; 
		const char *name; 
		entry_ref e; 
		
		if(entry.GetPath(&path) != B_NO_ERROR) 
			continue; 
		name = path.Path(); 
		
		if(entry.GetRef(&e) != B_NO_ERROR) 
			continue; 

		if(entry.IsDirectory()) { 
			if(strcmp(e.name, "floppy") == 0) 
				continue; // ignore floppy (it is not silent) 
			int devfd = try_dir(name);
			if(devfd >= 0)
				return devfd;
		} else { 
			int devfd; 
			device_geometry g; 

			if(strcmp(e.name, "raw") != 0) 
				continue; // ignore partitions 

			devfd = open(name, O_RDONLY); 
			if(devfd < 0) 
				continue; 

			if(ioctl(devfd, B_GET_GEOMETRY, &g, sizeof(g)) >= 0) {
				if(g.device_type == B_CD)
				{ 
					return devfd;
				}
			}
			close(devfd);
		} 
	}
	return B_ERROR;
}

/**********************************************************************
	RXgNV
**********************************************************************/
CCDPlayer_BeOS::CCDPlayer_BeOS()
{
	m_bEnd			= false;
	m_NowTrack= 0;
	m_NowLoop = 0;
	m_Devicefd = try_dir( "/dev/disk" );

	// XbhN
	m_ThreadID = ::spawn_thread( PlayFunc, "CCDPlayer_BeOS()", B_NORMAL_PRIORITY, this );	
	::resume_thread( m_ThreadID );
}

CCDPlayer_BeOS::~CCDPlayer_BeOS()
{
//	puts("~CCDPlayerBeOS()");
	m_bEnd = true;

	// XbhI
	if( m_ThreadID > 0){
		::snooze(500);
		::kill_thread( m_ThreadID );
	}
	StopTrack();
	if( m_Devicefd >= 0)::close( m_Devicefd);
}

/**********************************************************************
	Đ(gbNOŒ~)
**********************************************************************/
long CCDPlayer_BeOS::Play( long _Track, int _Loop )
{
	m_NowTrack = _Track;
	m_NowLoop  = _Loop;

	return 0;
}

/**********************************************************************
	gbN̎擾
**********************************************************************/
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
long CCDPlayer_BeOS::GetPlayTrack( ALSD::stCDPTrackInfo *_lpTrack )
#else
long CCDPlayer_BeOS::GetPlayTrack( stCDPTrackInfo *_lpTrack )
#endif
{
	return 0;
}

/**********************************************************************
	ĐThread
**********************************************************************/
int32 CCDPlayer_BeOS::PlayFunc( void *_lpVoid )
{
	CCDPlayer_BeOS *lpCDP = (CCDPlayer_BeOS *)_lpVoid;

	// foCX̃I[v

	long track, loop;
	long NowTrack = 0;
	while(! lpCDP->IsDestroy() ){
		::snooze(100);
		// bN

		// 
		track = lpCDP->GetNowTrack();
		loop  = lpCDP->GetNowLoop();
		// ݂̍ĐԂ𒲂ׂ
		CDState state = lpCDP->GetState();
		if( state == Stopped){
			// ~Ԃ...
			if( track != 0 && loop  >= 0 ){
				// Đ
				lpCDP->PlayTrack( track );
				//
				NowTrack = track;
				if( loop == 1 )		lpCDP->SetNowLoop( -1 );
				else if( loop > 0 )	lpCDP->SetNowLoop( loop-1 );
			}
		}else if( state == Playing ){
			// OȂ
			if( track == 0 ){
				// ~
				lpCDP->StopTrack();
				NowTrack = track;
			}else {
				// ݂ƈႤgbNIĂ邩H
				if( track != NowTrack ){
					// ʃgbNĐ
					lpCDP->PlayTrack( track );
					//
					NowTrack = track;
					if( loop == 1 )		lpCDP->SetNowLoop( -1 );
					else if( loop > 0 )	lpCDP->SetNowLoop( loop-1 );
				}
			}
		}
	}
	lpCDP->StopTrack();
}

/**********************************************************************
	Ԃ̎擾
**********************************************************************/
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
ALSD_CDP_BeOS::CDState CCDPlayer_BeOS::GetState( void )
#else
CDState CCDPlayer_BeOS::GetState( void )
#endif
{
	if( m_Devicefd == B_ERROR )return NoCD;

	// CD邩H
	status_t media_status = B_DEV_NO_MEDIA;
	::ioctl(m_Devicefd, B_GET_MEDIA_STATUS, &media_status, sizeof(media_status));
	if (media_status != B_NO_ERROR){
		return NoCD;
	}

	// ʒu擾
	scsi_position pos;
	status_t result = ::ioctl(m_Devicefd,  B_SCSI_GET_POSITION, &pos);

	//
	if( result != B_NO_ERROR ){
		//
		return NoCD;
	}else if( (!pos.position[1]) || (pos.position[1] >= 0x13) ||
		((pos.position[1] == 0x12) && (!pos.position[6])) ){
		//
		return Stopped;
	}else if (pos.position[1] == 0x11){
		//
		return Playing;
	}else{
		//
		return Paused;
	}

	return NoCD;
}

/**********************************************************************
	Đ̒~
**********************************************************************/
void CCDPlayer_BeOS::StopTrack( void )
{
//	puts( "StopTrack");
	status_t result = ::ioctl( m_Devicefd, B_SCSI_STOP_AUDIO);
	if (result != B_NO_ERROR) {
		TRACE("error %s playing track\n", strerror(errno));
	}
}

/**********************************************************************
	gbNĐ
**********************************************************************/
void CCDPlayer_BeOS::PlayTrack( long _Track)
{
//	printf("PlayTrack(%d)\n", _Track);
	scsi_play_track track;

	track.start_track = _Track;
	track.start_index = 1;
	track.end_track = _Track;
	track.end_index = 1;

	status_t result = ::ioctl( m_Devicefd, B_SCSI_PLAY_TRACK, &track);
	if (result != B_NO_ERROR) {
		TRACE("error %s playing track\n", strerror(errno));
	}
}
